import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { urlSlug } from 'src/utils/common-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { Services } from '../services/schema/services.schema';
import { ServicesRepository } from '../services/services.repository';
import { SolutionsServices } from '../solutions-services/schema/solutions-services.schema';
import { SolutionsServicesRepository } from '../solutions-services/solutions-services.repository';
import { CreateSolutionsDto } from './dto/create-solutions.dto';
import { UpdateSolutionsDto } from './dto/update-solutions.dto';
import { Solutions } from './schema/solutions.schema';
import { SolutionsRepository } from './solutions.repository';



@Injectable()
export class SolutionsService {
    constructor(private readonly doSpaceService: DoSpacesService,
        @InjectModel('services') private readonly servicesModel: Model<Services>,
        @InjectModel('solutions') private readonly solutionsModel: Model<Solutions>,
        @InjectModel('solutions-services') private readonly SolutionsServicesModel: Model<SolutionsServices>,
    ) { }

    private readonly solutionsServicesRepository = new SolutionsServicesRepository(this.SolutionsServicesModel);

    private readonly solutionsRepository = new SolutionsRepository(this.solutionsModel);

    private readonly servicesRepository = new ServicesRepository(this.servicesModel);

    async create(
        dto: CreateSolutionsDto,
        files: {
            image?: UploadedMulterFileI,
            icon?: UploadedMulterFileI
        }
    ): Promise<Solutions> {

        dto.urlSlug = urlSlug(dto.title);

        const res = await this.solutionsRepository.findByFilterQuery({ urlSlug: dto.urlSlug });
        if (Object.keys(res).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Solutions/");
            dto.image = img;
        }
        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "Solutions/");
            dto.icon = icon;
        }

        const data = await this.solutionsRepository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED,
            );
        }

        return ResponseUtils.successResponseHandler(201, "Successfully added to Solutions.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<Solutions[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.solutionsRepository.paginate(page, limit);

        // const solutionList = [];
        // for (const solution of data?.data) {
        //     const foundService = await this.solutionsServicesRepository.findOneEntity(solution?._id.toString());
        //     if (foundSolution) {
        //         const solutionObj = {
        //             solution: foundSolution,
        //             service: service
        //         }
        //         serviceList.push(solutionObj);
        //     }
        // }
        // if (!data) {
        //     throw new NotFoundException(
        //         Constants.NOT_FOUND,
        //     );
        // }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<Solutions | null> {
        const data = await this.solutionsRepository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateSolutionsDto,
        files: {
            image?: UploadedMulterFileI,
            icon?: UploadedMulterFileI
        }): Promise<Solutions | null> {
        const res = await this.solutionsRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }
        const foundImage = (res as Solutions)?.image;
        dto.urlSlug = urlSlug(dto.title);

        const resSlug = await this.solutionsRepository.findByFilterQuery({ urlSlug: dto.urlSlug, _id: { $ne: id } });
        if (Object.keys(resSlug).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Solutions/");
            dto.image = img;
        } else {
            dto.image = foundImage;
        }

        const foundIcon = (res as Solutions)?.icon;
        if (files && files.icon) {
            const img: any = await this.doSpaceService.uploadFile(files.icon[0], "Solutions/");
            dto.icon = img;
        } else {
            dto.icon = foundIcon;
        }

        const data = await this.solutionsRepository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED,
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.solutionsRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }

        const data = await this.solutionsRepository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED,
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}